@@ -22,8 +22,8 @@
  *
  */
 
-#include <errno.h>
 #include <features.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/syscall.h>
 
@@ -32,17 +32,16 @@
 /* Do not include unistd.h, so gcc doesn't whine about 
  * _exit returning.  It really doesn't return... */
 #define __NR__exit __NR_exit
-#ifdef __STR_NR_exit
-#define __STR_NR__exit __STR_NR_exit
-#endif
 _syscall1(void, _exit, int, status);
 #endif
 
 //#define __NR_fork             2
-#ifdef L_fork
+#ifdef L___libc_fork
 #include <unistd.h>
 #	ifdef __UCLIBC_HAS_MMU__
-		_syscall0(pid_t, fork);
+#define __NR___libc_fork __NR_fork
+		_syscall0(pid_t, __libc_fork);
+		weak_alias (__libc_fork, fork)
 #	else
 		pid_t fork(void)
 		{
@@ -53,49 +52,36 @@ _syscall1(void, _exit, int, status);
 #endif
 
 //#define __NR_read             3
-#ifdef L_read
+#ifdef L___libc_read
 #include <unistd.h>
-_syscall3(ssize_t, read, int, fd, __ptr_t, buf, size_t, count);
+#define __NR___libc_read __NR_read
+_syscall3(ssize_t, __libc_read, int, fd, __ptr_t, buf, size_t, count);
+weak_alias(__libc_read, read)
 #endif
 
 //#define __NR_write            4
-#ifdef L_write
+#ifdef L___libc_write
 #include <unistd.h>
-_syscall3(ssize_t, write, int, fd, const __ptr_t, buf, size_t, count);
-weak_alias(write, __write);
+#define __NR___libc_write __NR_write
+_syscall3(ssize_t, __libc_write, int, fd, const __ptr_t, buf, size_t, count);
+weak_alias(__libc_write, write)
 #endif
 
 //#define __NR_open             5
-#ifdef L___open
+#ifdef L___libc_open
 #include <stdarg.h>
-#include <fcntl.h>
-#define __NR___open __NR_open
-#ifdef __STR_NR_open
-#define __STR_NR___open __STR_NR_open
-#endif
-_syscall3(int, __open, const char *, fn, int, flags, mode_t, mode);
-
-int open(const char *file, int oflag, ...)
-{
-	int mode = 0;
-
-	if (oflag & O_CREAT) {
-		va_list args;
-
-		va_start(args, oflag);
-		mode = va_arg(args, int);
-
-		va_end(args);
-	}
-
-	return __open(file, oflag, mode);
-}
+/* Do not include fcntl.h, so gcc doesn't whine the prototype */
+#define __NR___libc_open __NR_open
+_syscall3(int, __libc_open, const char *, fn, int, flags, mode_t, mode);
+weak_alias(__libc_open, open)
 #endif
 
 //#define __NR_close            6
-#ifdef L_close
+#ifdef L___libc_close
 #include <unistd.h>
-_syscall1(int, close, int, fd);
+#define __NR___libc_close __NR_close
+_syscall1(int, __libc_close, int, fd);
+weak_alias(__libc_close, close)
 #endif
 
 //#define __NR_waitpid          7
@@ -182,15 +168,20 @@ _syscall3(int, lchown, const char *, path, uid_t, owner, gid_t, group);
 //#define __NR_oldstat          18
 
 //#define __NR_lseek            19
-#ifdef L_lseek
+#ifdef L___libc_lseek
 #include <unistd.h>
-_syscall3(off_t, lseek, int, fildes, off_t, offset, int, whence);
+#define __NR___libc_lseek __NR_lseek
+_syscall3(__off_t, __libc_lseek, int, fildes, __off_t, offset, int, whence);
+weak_alias(__libc_lseek, lseek)
 #endif
 
 //#define __NR_getpid           20
-#ifdef L_getpid
+#ifdef L___libc_getpid
 #include <unistd.h>
-_syscall0(pid_t, getpid);
+#define __NR___libc_getpid __NR_getpid
+_syscall0(pid_t, __libc_getpid);
+weak_alias(__libc_getpid, getpid)
+weak_alias(__libc_getpid, __getpid)
 #endif
 
 //#define __NR_mount            21
@@ -239,9 +230,6 @@ _syscall1(int, stime, const time_t *, t);
 #ifdef L___ptrace
 #include <sys/ptrace.h>
 #define __NR___ptrace __NR_ptrace
-#ifdef __STR_NR_ptrace
-#define __STR_NR___ptrace __STR_NR_ptrace
-#endif
 _syscall4(long, __ptrace, enum __ptrace_request, request, pid_t, pid,
 		void*, addr, void*, data);
 #endif
@@ -256,9 +244,11 @@ _syscall1(unsigned int, alarm, unsigned int, seconds);
 //#define __NR_oldfstat         28
 
 //#define __NR_pause            29
-#ifdef L_pause
+#ifdef L___libc_pause
 #include <unistd.h>
-_syscall0(int, pause);
+#define __NR___libc_pause __NR_pause
+_syscall0(int, __libc_pause);
+weak_alias(__libc_pause, pause)
 #endif
 
 //#define __NR_utime            30
@@ -341,13 +331,8 @@ _syscall1(int, dup, int, oldfd);
 //#define __NR_pipe             42
 #ifdef L_pipe
 #include <unistd.h>
-/*
- * SH has a weird register calling mechanism for pipe, see pipe.c
- */
-#if !defined(__sh__)
 _syscall1(int, pipe, int *, filedes);
 #endif
-#endif
 
 //#define __NR_times            43
 #ifdef L_times
@@ -426,9 +411,6 @@ _syscall1(int, acct, const char *, filename);
 #include <stdarg.h>
 #include <sys/ioctl.h>
 #define __NR__ioctl __NR_ioctl
-#ifdef __STR_NR_ioctl
-#define __STR_NR__ioctl __STR_NR_ioctl
-#endif
 extern int _ioctl(int fd, int request, void *arg);
 
 _syscall3(int, _ioctl, int, fd, int, request, void *, arg);
@@ -451,14 +433,11 @@ int ioctl(int fd, unsigned long int request, ...)
 #include <stdarg.h>
 #include <fcntl.h>
 #define __NR__fcntl __NR_fcntl
-#ifdef __STR_NR_fcntl
-#define __STR_NR__fcntl __STR_NR_fcntl
-#endif
 extern int _fcntl(int fd, int cmd, long arg);
 
 _syscall3(int, _fcntl, int, fd, int, cmd, long, arg);
 
-int fcntl(int fd, int command, ...)
+int __libc_fcntl(int fd, int command, ...)
 {
 	long arg;
 	va_list list;
@@ -469,6 +448,7 @@ int fcntl(int fd, int command, ...)
 	va_end(list);
 	return _fcntl(fd, command, arg);
 }
+weak_alias(__libc_fcntl, fcntl)
 #endif
 
 //#define __NR_mpx              56
@@ -663,9 +643,6 @@ _syscall2(int, swapon, const char *, path, int, swapflags);
 //#define __NR_reboot           88
 #ifdef L__reboot
 #define __NR__reboot __NR_reboot
-#ifdef __STR_NR_reboot
-#define __STR_NR__reboot __STR_NR_reboot
-#endif
 extern int _reboot(int magic, int magic2, int flag);
 
 _syscall3(int, _reboot, int, magic, int, magic2, int, flag);
@@ -681,9 +658,6 @@ int reboot(int flag)
 //#define __NR_mmap             90
 #ifdef L__mmap
 #define __NR__mmap __NR_mmap
-#ifdef __STR_NR_mmap
-#define __STR_NR__mmap __STR_NR_mmap
-#endif
 #include <unistd.h>
 #include <sys/mman.h>
 extern __ptr_t _mmap(unsigned long *buffer);
@@ -715,13 +689,13 @@ _syscall2(int, munmap, void *, start, size_t, length);
 //#define __NR_truncate         92
 #ifdef L_truncate
 #include <unistd.h>
-_syscall2(int, truncate, const char *, path, off_t, length);
+_syscall2(int, truncate, const char *, path, __off_t, length);
 #endif
 
 //#define __NR_ftruncate        93
 #ifdef L_ftruncate
 #include <unistd.h>
-_syscall2(int, ftruncate, int, fd, off_t, length);
+_syscall2(int, ftruncate, int, fd, __off_t, length);
 #endif
 
 //#define __NR_fchmod           94
@@ -764,7 +738,6 @@ _syscall2(int, fstatfs, int, fd, struct statfs *, buf);
 
 //#define __NR_ioperm           101
 #ifdef L_ioperm
-#include <sys/io.h>
 #	if defined __UCLIBC_HAS_MMU__ && defined __NR_ioperm
 		_syscall3(int, ioperm, unsigned long, from, unsigned long, num, int, turn_on);
 #	else
@@ -785,9 +758,6 @@ _syscall2(int, socketcall, int, call, unsigned long *, args);
 #ifdef L__syslog
 #include <unistd.h>
 #define __NR__syslog		__NR_syslog
-#ifdef __STR_NR_syslog
-#define __STR_NR__syslog	__STR_NR_syslog
-#endif
 extern int _syslog(int type, char *buf, int len);
 
 _syscall3(int, _syslog, int, type, char *, buf, int, len);
@@ -817,9 +787,6 @@ _syscall2(int, getitimer, __itimer_which_t, which, struct itimerval *, value);
 #include <unistd.h>
 #include "statfix.h"
 #define __NR___stat	__NR_stat
-#ifdef __STR_NR_stat
-#define __STR_NR___stat	__STR_NR_stat
-#endif
 extern int __stat(const char *file_name, struct kernel_stat *buf);
 _syscall2(int, __stat, const char *, file_name, struct kernel_stat *, buf);
 
@@ -845,9 +812,6 @@ int stat(const char *file_name, struct libc_stat *buf)
 #include <unistd.h>
 #include "statfix.h"
 #define __NR___lstat	__NR_lstat
-#ifdef __STR_NR_lstat
-#define __STR_NR___lstat	__STR_NR_lstat
-#endif
 extern int __lstat(const char *file_name, struct kernel_stat *buf);
 _syscall2(int, __lstat, const char *, file_name, struct kernel_stat *, buf);
 
@@ -873,9 +837,6 @@ int lstat(const char *file_name, struct libc_stat *buf)
 #include <unistd.h>
 #include "statfix.h"
 #define __NR___fstat	__NR_fstat
-#ifdef __STR_NR_fstat
-#define __STR_NR___fstat	__STR_NR_fstat
-#endif
 extern int __fstat(int filedes, struct kernel_stat *buf);
 _syscall2(int, __fstat, int, filedes, struct kernel_stat *, buf);
 
@@ -900,7 +861,6 @@ int fstat(int filedes, struct libc_stat *buf)
 
 //#define __NR_iopl             110
 #ifdef L_iopl
-#include <sys/io.h>
 /* Tuns out the m68k unistd.h kernel header is broken */
 #	if defined __UCLIBC_HAS_MMU__ && defined __NR_iopl && ! defined(__mc68000__)
 		_syscall1(int, iopl, int, level);
@@ -944,16 +904,15 @@ _syscall1(int, sysinfo, struct sysinfo *, info);
 //#define __NR_ipc              117
 #ifdef L___ipc
 #define __NR___ipc __NR_ipc
-#ifdef __STR_NR_ipc
-#define __STR_NR___ipc __STR_NR_ipc
-#endif
 _syscall5(int, __ipc, unsigned int, call, int, first, int, second, int, third, void *, ptr);
 #endif
 
 //#define __NR_fsync            118
-#ifdef L_fsync
+#ifdef L___libc_fsync
 #include <unistd.h>
-_syscall1(int, fsync, int, fd);
+#define __NR___libc_fsync __NR_fsync
+_syscall1(int, __libc_fsync, int, fd);
+weak_alias(__libc_fsync, fsync)
 #endif
 
 //#define __NR_sigreturn        119
@@ -980,6 +939,7 @@ _syscall1(int, uname, struct utsname *, buf);
 #ifdef L_adjtimex
 #include <sys/timex.h>
 _syscall1(int, adjtimex, struct timex *, buf);
+weak_alias(adjtimex, __adjtimex);
 #endif
 
 //#define __NR_mprotect         125
@@ -1078,23 +1038,25 @@ _syscall1(int, setfsgid, gid_t, gid);
 
 //#define __NR__llseek          140
 #ifdef L__llseek
-extern int _llseek(int fd, off_t hoff, off_t loff, loff_t *res, int whence);
+#ifdef __UCLIBC_HAVE_LFS__
+extern int _llseek(int fd, __off_t offset_hi, __off_t offset_lo, 
+		__loff_t *result, int whence);
 
-_syscall5(int, _llseek, int, fd, off_t, hoff, off_t, loff, loff_t *, res,
-		  int, whence);
+_syscall5(int, _llseek, int, fd, __off_t, offset_hi, __off_t, offset_lo, 
+		__loff_t *, result, int, whence);
 
-loff_t llseek(int fd, loff_t offset, int whence)
+__loff_t __libc_lseek64(int fd, __loff_t offset, int whence)
 {
 	int ret;
-	loff_t result;
+	__loff_t result;
 
-	ret = _llseek(fd, (off_t) (offset >> 32),
-				  (off_t) (offset & 0xffffffff), &result, whence);
+	ret = _llseek(fd, (__off_t) (offset >> 32),
+				  (__off_t) (offset & 0xffffffff), &result, whence);
 
-	return ret ? (loff_t) ret : result;
+	return ret ? (__loff_t) ret : result;
 }
-#ifdef __UCLIBC_HAVE_LFS__
-weak_alias(llseek, lseek64);
+weak_alias(__libc_lseek64, llseek);
+weak_alias(__libc_lseek64, lseek64);
 #endif
 #endif
 
@@ -1122,8 +1084,15 @@ _syscall2(int,flock,int,fd, int,operation);
 #endif
 
 //#define __NR_msync            144
-//#define __NR_readv            145
+#ifdef L___libc_msync
+#include <unistd.h>
+#include <sys/mman.h>
+#define __NR___libc_msync __NR_msync
+_syscall3(int, __libc_msync, void *, addr, size_t, length, int, flags);
+weak_alias(__libc_msync, msync);
+#endif
 
+//#define __NR_readv            145
 #ifdef L_readv
 #include <sys/uio.h>
 _syscall3(ssize_t, readv, int, filedes, const struct iovec *, vector, int,
@@ -1250,17 +1219,18 @@ _syscall2(int, sched_rr_get_interval, pid_t, pid, struct timespec *, tp);
 #endif
 
 //#define __NR_nanosleep                162
-#ifdef L_nanosleep
+#ifdef L___libc_nanosleep
 #include <time.h>
-_syscall2(int, nanosleep, const struct timespec *, req, struct timespec *, rem);
+#define __NR___libc_nanosleep __NR_nanosleep
+_syscall2(int, __libc_nanosleep, const struct timespec *, req, struct timespec *, rem);
+weak_alias(__libc_nanosleep, nanosleep)
 #endif
 
 //#define __NR_mremap                   163
 #ifdef L_mremap
 #include <unistd.h>
 #include <sys/mman.h>
-_syscall4(__ptr_t, mremap, __ptr_t, old_address, size_t, old_size, size_t,
-		  new_size, int, may_move);
+_syscall4(__ptr_t, mremap, __ptr_t, old_address, size_t, old_size, size_t, new_size, int, may_move);
 #endif
 
 //#define __NR_setresuid                164
@@ -1357,7 +1327,23 @@ int sigsuspend (const sigset_t *mask)
 #endif
 
 //#define __NR_pread                    180
+#ifdef L___libc_pread
+#define _XOPEN_SOURCE 500
+#include <unistd.h>
+#define __NR___libc_pread __NR_pread
+_syscall4(ssize_t, __libc_pread, int, fd, void *, buf, size_t, count, __off_t, offset);
+weak_alias (__libc_pread, pread)
+#endif
+
 //#define __NR_pwrite                   181
+#ifdef L___libc_pwrite
+#define _XOPEN_SOURCE 500
+#include <unistd.h>
+#define __NR___libc_pwrite __NR_pwrite
+_syscall4(ssize_t, __libc_pwrite, int, fd, const void *, buf, size_t, count, __off_t, offset);
+weak_alias (__libc_pwrite, pwrite)
+#endif
+
 //#define __NR_chown                    182
 #ifdef L_chown
 #include <unistd.h>
@@ -1406,7 +1392,7 @@ _syscall2(int, sigaltstack, const struct sigaltstack *, ss, struct sigaltstack *
 #ifdef L_sendfile
 #include <unistd.h>
 #include <sys/sendfile.h>
-_syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, off_t *, offset, size_t, count)
+_syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, __off_t *, offset, size_t, count)
 #endif
 #endif
 
@@ -1422,31 +1408,42 @@ _syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, off_t *, offset, size_t, co
 
 
 //#define __NR_truncate64         193
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L_truncate64
+#ifdef __UCLIBC_HAVE_LFS__
+#include <bits/wordsize.h>
+/* Using _syscall2 to pass 64-bit arguments generally only works on 64-bit 
+ * systems, so we only implement truncate64/ftruncate64 in that case.  Ports 
+ * for processors with shorter word-lengths should define their own custom 
+ * versions instead.  */
+#if __WORDSIZE >= 64
 #include <unistd.h>
 _syscall2(int, truncate64, const char *, path, __off64_t, length);
-#endif
+#endif /* __WORDSIZE >= 64 */
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 //#define __NR_ftruncate64        194
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L_ftruncate64
+#ifdef __UCLIBC_HAVE_LFS__
+#include <bits/wordsize.h>
+/* Using _syscall2 to pass 64-bit arguments generally only works on 64-bit 
+ * systems, so we only implement truncate64/ftruncate64 in that case.  Ports 
+ * for processors with shorter word-lengths should define their own custom 
+ * versions instead.  */
+#if __WORDSIZE >= 64
 #include <unistd.h>
 _syscall2(int, ftruncate64, int, fd, __off64_t, length);
-#endif
+#endif /* __WORDSIZE >= 64 */
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 
 //#define __NR_stat64             195
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L___stat64
+#ifdef __UCLIBC_HAVE_LFS__
 #include <unistd.h>
 #include "statfix64.h"
 #define __NR___stat64	__NR_stat64
-#ifdef __STR_NR_stat64
-#define __STR_NR___stat64	__STR_NR_stat64
-#endif
 extern int __stat64(const char *file_name, struct kernel_stat64 *buf);
 _syscall2(int, __stat64, const char *, file_name, struct kernel_stat64 *, buf);
 
@@ -1465,18 +1462,15 @@ int stat64(const char *file_name, struct libc_stat64 *buf)
 {
 	return(__xstat64(0, file_name, buf));
 }
-#endif
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 //#define __NR_lstat64            196
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L___lstat64
+#ifdef __UCLIBC_HAVE_LFS__
 #include <unistd.h>
 #include "statfix64.h"
 #define __NR___lstat64	__NR_lstat64
-#ifdef __STR_NR_lstat64
-#define __STR_NR___lstat64	__STR_NR_lstat64
-#endif
 extern int __lstat64(const char *file_name, struct kernel_stat64 *buf);
 _syscall2(int, __lstat64, const char *, file_name, struct kernel_stat64 *, buf);
 
@@ -1495,18 +1489,15 @@ int lstat64(const char *file_name, struct libc_stat64 *buf)
 {
 	return(__lxstat64(0, file_name, buf));
 }
-#endif
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 //#define __NR_fstat64            197
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L___fstat64
+#ifdef __UCLIBC_HAVE_LFS__
 #include <unistd.h>
 #include "statfix64.h"
 #define __NR___fstat64	__NR_fstat64
-#ifdef __STR_NR_fstat64
-#define __STR_NR___fstat64	__STR_NR_fstat64
-#endif
 extern int __fstat64(int filedes, struct kernel_stat64 *buf);
 _syscall2(int, __fstat64, int, filedes, struct kernel_stat64 *, buf);
 
@@ -1525,8 +1516,8 @@ int fstat64(int filedes, struct libc_stat64 *buf)
 {
 	return(__fxstat64(0, filedes, buf));
 }
-#endif
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 
 //#define __NR_lchown32		198
@@ -1560,18 +1551,18 @@ _syscall2(int, pivot_root, const char *, new_root, const char *, put_old)
 //#define __NR_madvise1		219	/* delete when C lib stub is removed */
 
 //#define __NR_getdents64		220
-#ifdef __UCLIBC_HAVE_LFS__
 #ifdef L_getdents64
+#ifdef __UCLIBC_HAVE_LFS__
 #include <unistd.h>
 #include <dirent.h>
 _syscall3(int, getdents64, int, fd, char *, dirp, size_t, count);
-#endif
 #endif /* __UCLIBC_HAVE_LFS__ */
+#endif
 
 //#define __NR_fcntl64		221
+#ifdef L__fcntl64
 #ifdef __UCLIBC_HAVE_LFS__
 #define __NR__fcntl64 __NR_fcntl64
-#ifdef L__fcntl64
 #include <stdarg.h>
 #include <fcntl.h>
 extern int _fcntl64(int fd, int cmd, long arg);
